<?php

class HomeController extends BaseController {

	/*
	|--------------------------------------------------------------------------
	| Default Home Controller
	|--------------------------------------------------------------------------
	|
	| You may wish to use controllers instead of, or in addition to, Closure
	| based routes. That's great! Here is an example controller method to
	| get you started. To route to this controller, just add the route:
	|
	|	Route::get('/', 'HomeController@showWelcome');
	|
	*/


	// ********** Main Home Controller //

	public function home(){
		try{

			$settings = Setting::first();
			if($settings){
				
				$search = Input::get('search');
				if(isset($search)){
					$media = Media::where('active', '=', 1)->where('title', 'LIKE', '%'.$search.'%')->orderBy('created_at', 'desc')->paginate(Config::get('site.num_results_per_page'));
				} else {
					$media = Media::where('active', '=', 1)->orderBy('created_at', 'desc')->paginate(Config::get('site.num_results_per_page'));
				}

				$data = array(
					'media' => $media,
					'search' => $search
					);

				return View::make('home', $data);

			} else {
				throw new Exception('settings not set, first install the script');
			}

		}catch(Exception $e){

			return Redirect::to('install');

		}
	}


	// ********** Main Install Route ********** //

	public function install(){
		try{

			$settings = Setting::first();
			if($settings){
				return Redirect::to('/');
			} else {
				throw new Exception('settings not set, first install the script');
			}

		}catch(Exception $e){

				return View::make('install');

		}
	}


	// ********** Add the DB Credentials to a file ********** //

	public function add_db_credentials(){
		if(Request::ajax()){

			try{

				$settings = Setting::first();
				if($settings){
					return Redirect::to('/');
				} else {
					throw new Exception('settings not set, first install the script');
				}

			}catch(Exception $e){

				$db_host = $_POST['database_host'];
				$db_name = $_POST['database_name'];
				$db_user = $_POST['database_user'];
				$db_password = $_POST['database_password'];
				
				$file = '../app/config/config.php';
				$config_string = "<?php return array('db_host' => '$db_host', 'db_name' => '$db_name', 'db_user' => '$db_user', 'db_password' => '$db_password');";
				$new_file = file_put_contents($file, $config_string);

				if($new_file !== FALSE){
					echo true;
				} else {
					echo false;
				}
			}

		} else {
			echo false;
		}
	}

	// ********** Test the Database Connection ********** //

	public function test_db_connection(){
		if(Request::ajax()){

			try{

				$db_host = $_POST['database_host'];
				$db_name = $_POST['database_name'];
				$db_user = $_POST['database_user'];
				$db_password = $_POST['database_password'];

				// Create connection
				$con=mysqli_connect($db_host,$db_user,$db_password,$db_name);

				// Check connection
				if (mysqli_connect_errno()){
					echo false;
				} else {
					echo true;
				}

			} catch (Exception $e){
				echo false;
			}

		} else {
			echo false;
		}
	}

	// ********** Add pre-installed data ********** //

	public function add_db_data(){
		if(Request::ajax()){
			try{

				$settings = Setting::first();
				if($settings){
					return Redirect::to('/');
				} else {
					throw new Exception('settings not set, first install the script');
				}

			}catch(Exception $e){

				try{

					$db_host = $_POST['database_host'];
					$db_name = $_POST['database_name'];
					$db_user = $_POST['database_user'];
					$db_password = $_POST['database_password'];

					Config::set('database.connections.mysql.host', $db_host);
					Config::set('database.connections.mysql.database', $db_name);
					Config::set('database.connections.mysql.username', $db_user);
					Config::set('database.connections.mysql.password', $db_password);
					
					Artisan::call('migrate');
					Artisan::call('db:seed', array('--class'=> "SettingsTableSeeder"));
					Artisan::call('db:seed', array('--class'=> "CategoriesTableSeeder"));

					if($_POST['preloaded_data'] == 'true'){
						Artisan::call('db:seed', array('--class'=> "MediaTableSeeder"));
					}

					$this->upgrade();

					echo true;
				
				} catch(Exception $e){
					echo false;
				}

			}

		} else {
			echo false;
		}
	}

	// ********** Add the admin to the database ********** //

	public function add_admin_user(){
		if(Request::ajax()){

			try{

				$user = User::first();
				if($user){
					return Redirect::to('/');
				} else {
					throw new Exception('We cannot detect any current user. Okay to add new admin');
				}

			}catch(Exception $e){

				$admin_username = Input::get('admin_username');
				$admin_email = Input::get('admin_email');
				$admin_password = Hash::make(Input::get('admin_password'));

				$user = new User;
				$user->username = $admin_username;
				$user->email = $admin_email;
				$user->password = $admin_password;
				$user->admin = 1;

				$new_user = $user->save();

				$point = new Point;
		    	$point->user_id = $user->id;
		    	$point->points = 200;
		    	$point->description = Lang::get('users.registration');
		    	$point->save();

				if($new_user){
					Auth::attempt(array('email' => Input::get('admin_email'), 'password' => Input::get('admin_password')));
					echo true;
				} else {
					echo false;
				}

			}

		} else {
			echo false;
		}
	}

	// ********** Upgrade functionality ********** //

	public function upgrade(){

		$upgraded = $this->upgrade102();
		$upgraded = $this->upgrade105();
		$upgraded = $this->upgrade106();
		$upgraded = $this->upgrade107();
		$upgraded = $this->upgrade108();
		$upgraded = $this->upgrade109();

		if($upgraded){
			return Redirect::to('/')->with(array('note' => 'Successfully Updated Your Script', 'note_type' => 'success') );
		} else {
			return Redirect::to('/');
		}

	}

	public function upgrade102(){

		if( !Schema::hasColumn('settings', 'analytics') ){

			Schema::table('settings', function($table)
			{
			    // Added for V 1.0.2
				$table->text('analytics');
				$table->boolean('user_registration')->default(1);
				$table->boolean('infinite_scroll')->default(1);
			});

			Schema::table('users', function($table){
				$table->string('activation_code')->nullable();
			});

			return true;

		} else {
			return false;
		}
	}

	public function upgrade105(){

		if( !Schema::hasColumn('settings', 'random_bar_enabled') ){

			Schema::table('settings', function($table)
			{
			    // Added for V 1.0.5
				$table->boolean('random_bar_enabled')->default(0);
			});

			return true;

		} else {
			return false;
		}

	}

	public function upgrade106(){

		if(!Schema::hasColumn('settings', 'media_description')){

			Schema::table('settings', function($table)
			{
			    // Added for V 1.0.6
				$table->boolean('media_description')->default(0);
			});

			return true;

		} else {
			return false;
		}

	}

	public function upgrade107(){

		if(!Schema::hasColumn('settings', 'inbetween_ads')){

			Schema::table('settings', function($table)
			{
			    // Added for V 1.0.7
				$table->text('inbetween_ads')->nullable();
				$table->integer('inbetween_ads_repeat')->default(5);
			});

			Schema::table('media', function($table)
			{
			    // Added for V 1.0.7
				$table->boolean('nsfw')->default(0);
			});

			return true;

		} else {
			return false;
		}

	}

	public function upgrade108(){

		if(!Schema::hasColumn('settings', 'enable_watermark')){

			Schema::table('settings', function($table)
			{
			    // Added for V 1.0.8
				$table->boolean('enable_watermark')->default(0);
				$table->string('watermark_image')->default('assets/img/watermark.png');
				$table->string('watermark_position')->default('bottom-right');
				$table->integer('watermark_offset_x')->default(0);
				$table->integer('watermark_offset_y')->default(0);
			});

			return true;

		} else {
			return false;
		}

	}

	public function upgrade109(){

		$return_value = false;

		if(!Schema::hasColumn('media', 'views')){

			Schema::table('media', function($table)
			{
			    // Added for V 1.0.9
				$table->integer('views')->default(0);
				//update description to text
				DB::statement('ALTER TABLE `media` CHANGE `description` `description` TEXT');
			});

			$return_value = true;
		}

		if(!Schema::hasColumn('settings', 'pages_in_menu')){

			Schema::table('settings', function($table)
			{
				$table->boolean('pages_in_menu')->default(1);
			});

			$return_value = true;
		}

		if(!Schema::hasColumn('settings', 'pages_in_menu_text')){

			Schema::table('settings', function($table)
			{
				$table->string('pages_in_menu_text')->default('More');
			});

			$return_value = true;
		}

		if(!Schema::hasColumn('settings', 'infinite_load_btn')){

			Schema::table('settings', function($table)
			{
				$table->boolean('infinite_load_btn')->default(0);
			});

			$return_value = true;
		}

		if (!Schema::hasTable('pages'))
		{
		    Schema::create('pages', function($table) {
				$table->increments('id');
				$table->string('title');
				$table->string('url');
				$table->text('body');
				$table->integer('order')->default(0);
				$table->boolean('active')->default(1);
				$table->boolean('show_in_menu')->default(1);
				$table->timestamps();
			});

			$return_value = true;
		}

		return $return_value;

	}

}